package id

import (
	"fmt"
)

const (
	Client        = "client"
	ClientsPrefix = "clients"
)

var _ Descriptor = &ClientId{}

type ClientId struct {
	SpaceId
	ClientID string `json:"client_id,omitempty" bson:"client_id,omitempty"`
}

func (id *ClientId) New() Descriptor {
	return &ClientId{}
}

func (id *ClientId) Type() string { return Client }

func (id *ClientId) String() string {
	return Join(id.SpaceId.String(), ClientsPrefix, id.ClientID)
}

func (id *ClientId) FromParts(parts []string) error {
	if len(parts) != 4 || parts[2] != ClientsPrefix {
		return ErrInvalidID
	}
	if err := id.SpaceId.FromParts(parts[:2]); err != nil {
		return err
	}
	id.ClientID = parts[3]
	return nil
}

func (id *ClientId) Map() map[string]any {
	m := id.SpaceId.Map()
	m["client_id"] = id.ClientID
	m["type"] = Client
	return m
}

func (id *ClientId) FromMap(m map[string]any) error {
	id.ClientID, _ = m["client_id"].(string)
	if id.ClientID == "" {
		return fmt.Errorf("%w: ClientID required", ErrInvalidID)
	}
	return id.SpaceId.FromMap(m)
}

func (id *ClientId) Validate() error {
	if id.ClientID == "" {
		return fmt.Errorf("%w: ClientID required", ErrInvalidID)
	}

	return id.SpaceId.Validate()
}

func NewClientId(spaceID, id string) *ObjectId {
	return &ObjectId{Descriptor: &ClientId{SpaceId: SpaceId{SpaceID: spaceID}, ClientID: id}}
}
